@@ -1,90 +1,57 @@
 """
-Zinc Sceneviewer Widget
+   Zinc Sceneviewer Widget
 
-Implements a Zinc Sceneviewer Widget on Python using PySide2,
-which renders the Zinc Scene with OpenGL and allows interactive
-transformation of the view.
-Widget is derived from QtOpenGL.QGLWidget.
+   Implements a Zinc Sceneviewer Widget on Python using PySide2,
+   which renders the Zinc Scene with OpenGL and allows interactive
+   transformation of the view.
+   Widget is derived from QtWidgets.QOpenGLWidget
 
-This Source Code Form is subject to the terms of the Mozilla Public
-License, v. 2.0. If a copy of the MPL was not distributed with this
-file, You can obtain one at http://mozilla.org/MPL/2.0/.
+   Copyright 2015 University of Auckland
+
+   Licensed under the Apache License, Version 2.0 (the "License");
+   you may not use this file except in compliance with the License.
+   You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 """
-# This python module is intended to facilitate users creating their own applications that use OpenCMISS-Zinc
-# See the examples at https://svn.physiomeproject.org/svn/cmiss/zinc/bindings/trunk/python/ for further
-# information.
+from PySide2 import QtCore, QtWidgets
 
-from PySide2 import QtCore, QtOpenGL
-
-# from opencmiss.zinc.glyph import Glyph
 from opencmiss.zinc.sceneviewer import Sceneviewer, Sceneviewerevent
 from opencmiss.zinc.sceneviewerinput import Sceneviewerinput
 from opencmiss.zinc.scenecoordinatesystem import \
-        SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT,\
-        SCENECOORDINATESYSTEM_WORLD
+    SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT, \
+    SCENECOORDINATESYSTEM_WORLD
 from opencmiss.zinc.field import Field
 from opencmiss.zinc.glyph import Glyph
 from opencmiss.zinc.result import RESULT_OK
 
-selection_group_name = 'cmiss_selection'
-
-# mapping from qt to zinc start
-# Create a button map of Qt mouse buttons to Zinc input buttons
-button_map = {QtCore.Qt.LeftButton: Sceneviewerinput.BUTTON_TYPE_LEFT,
-              QtCore.Qt.MidButton: Sceneviewerinput.BUTTON_TYPE_MIDDLE,
-              QtCore.Qt.RightButton: Sceneviewerinput.BUTTON_TYPE_RIGHT}
+from opencmiss.zincwidgets.definitions import ProjectionMode, SelectionMode, \
+    BUTTON_MAP, modifier_map, SELECTION_GROUP_NAME
 
 
-# Create a modifier map of Qt modifier keys to Zinc modifier keys
-def modifier_map(qt_modifiers):
-    """
-    Return a Zinc Sceneviewerinput modifiers object that is created from
-    the Qt modifier flags passed in.
-    """
-    modifiers = Sceneviewerinput.MODIFIER_FLAG_NONE
-    if qt_modifiers & QtCore.Qt.SHIFT:
-        modifiers = modifiers | Sceneviewerinput.MODIFIER_FLAG_SHIFT
-
-    return modifiers
-# mapping from qt to zinc end
-
-
-SELECTION_RUBBERBAND_NAME = 'selection_rubberband'
-
-
-# projectionMode start
-class ProjectionMode(object):
-
-    PARALLEL = 0
-    PERSPECTIVE = 1
-# projectionMode end
-
-
-# selectionMode start
-class SelectionMode(object):
-
-    NONE = -1
-    EXCLUSIVE = 0
-    ADDITIVE = 1
-# selectionMode end
-
-
-class SceneviewerWidget(QtOpenGL.QGLWidget):
-    
+class SceneviewerWidget(QtWidgets.QOpenGLWidget):
     graphicsInitialized = QtCore.Signal()
 
     # init start
-    def __init__(self, parent=None, shared=None):
+    def __init__(self, parent=None):
         """
         Call the super class init functions, set the  Zinc context and the scene viewer handle to None.
         Initialise other attributes that deal with selection and the rotation of the plane.
         """
-        super(SceneviewerWidget, self).__init__(parent, shared)
+        super(SceneviewerWidget, self).__init__(parent)
         # Create a Zinc context from which all other objects can be derived either directly or indirectly.
+        self._handle_mouse_events = True
         self._graphicsInitialized = False
         self._context = None
         self._sceneviewer = None
         self._scenepicker = None
+        self._use_zinc_mouse_event_handling = False
 
         # Selection attributes
         self._selectionKeyHandling = True  # set to False if parent widget is to handle selection key presses
@@ -95,7 +62,6 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         self._selectionBox = None  # created and destroyed on demand in mouse events
         self._selectionFilter = None  # Client-specified filter which is used in logical AND with sceneviewer filter in selection
         self._selectTol = 3.0  # how many pixels on all sides to add to selection box when a point is clicked on
-        self._ignore_mouse_events = False
         self._selectionKeyPressed = False
         self.setFocusPolicy(QtCore.Qt.StrongFocus)
         self._selection_position_start = None
@@ -111,10 +77,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
             self._createSceneviewer()
 
     def getContext(self):
-        if self._context is not None:
-            return self._context
-        else:
-            raise RuntimeError("Zinc context has not been set in Sceneviewerwidget.")
+        return self._context
 
     def _createSceneviewer(self):
         # Following throws exception if you haven't called setContext() yet
@@ -122,11 +85,12 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         self._sceneviewernotifier = None
 
         # From the scene viewer module we can create a scene viewer, we set up the
-        # scene viewer to have the same OpenGL properties as the QGLWidget.
+        # scene viewer to have the same OpenGL properties as the QOpenGLWidget.
         sceneviewermodule = self._context.getSceneviewermodule()
         self._sceneviewer = sceneviewermodule.createSceneviewer(Sceneviewer.BUFFERING_MODE_DOUBLE, Sceneviewer.STEREO_MODE_DEFAULT)
         self._sceneviewer.setProjectionMode(Sceneviewer.PROJECTION_MODE_PERSPECTIVE)
-        self._sceneviewer.setViewportSize(self.width(), self.height())
+        pixel_scale = self.window().devicePixelRatio()
+        self._sceneviewer.setViewportSize(self.width() * pixel_scale, self.height() * pixel_scale)
 
         # Get the default scene filter, which filters by visibility flags
         scenefiltermodule = self._context.getScenefiltermodule()
@@ -135,7 +99,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
 
         region = self._context.getDefaultRegion()
         scene = region.getScene()
-        self._sceneviewer.setScene(scene)
+        self.setScene(scene)
 
         self._sceneviewernotifier = self._sceneviewer.createSceneviewernotifier()
         self._sceneviewernotifier.setCallback(self._zincSceneviewerEvent)
@@ -172,7 +136,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         scene = self._sceneviewer.getScene()
         region = scene.getRegion()
         fieldmodule = region.getFieldmodule()
-        selectionGroup = fieldmodule.findFieldByName(selection_group_name)
+        selectionGroup = fieldmodule.findFieldByName(SELECTION_GROUP_NAME)
         if selectionGroup.isValid():
             selectionGroup = selectionGroup.castGroup()
             if selectionGroup.isValid():
@@ -180,7 +144,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         if not selectionGroup.isValid():
             fieldmodule.beginChange()
             selectionGroup = fieldmodule.createFieldGroup()
-            selectionGroup.setName(selection_group_name)
+            selectionGroup.setName(SELECTION_GROUP_NAME)
             fieldmodule.endChange()
         scene.setSelectionField(selectionGroup)
         return selectionGroup
@@ -196,7 +160,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         Get the scene viewer for this ZincWidget.
         """
         return self._sceneviewer
-    
+
     def setSelectionModeAdditive(self):
         self._selectionAlwaysAdditive = True
 
@@ -239,7 +203,6 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         self._dataSelectMode = True
         self._elemSelectMode = True
 
-    # initializeGL start
     def initializeGL(self):
         """
         The OpenGL context is ready for use. If Zinc Context has been set, create Zinc Sceneviewer, otherwise
@@ -249,7 +212,12 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         if self._context:
             self._createSceneviewer()
         self.graphicsInitialized.emit()
-        # initializeGL end
+
+    def sizeHint(self):
+        return QtCore.QSize(200, 400)
+
+    def minimumSizeHint(self):
+        return QtCore.QSize(20, 30)
 
     def setProjectionMode(self, mode):
         if mode == ProjectionMode.PARALLEL:
@@ -267,7 +235,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         result, eye, lookat, up = self._sceneviewer.getLookatParameters()
         if result == RESULT_OK:
             angle = self._sceneviewer.getViewAngle()
-            return (eye, lookat, up, angle)
+            return eye, lookat, up, angle
 
         return None
 
@@ -319,7 +287,8 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         on the far clipping plane, and +1 is on the near clipping plane.
         """
         in_coords = [x, y, z]
-        result, out_coords = self._sceneviewer.transformCoordinates(SCENECOORDINATESYSTEM_WORLD, SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT, self._sceneviewer.getScene(), in_coords)
+        result, out_coords = self._sceneviewer.transformCoordinates(SCENECOORDINATESYSTEM_WORLD, SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT, self._sceneviewer.getScene(),
+                                                                    in_coords)
         if result == RESULT_OK:
             return out_coords  # [out_coords[0] / out_coords[3], out_coords[1] / out_coords[3], out_coords[2] / out_coords[3]]
 
@@ -333,7 +302,8 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         on the far clipping plane, and +1 is on the near clipping plane.
         """
         in_coords = [x, y, z]
-        result, out_coords = self._sceneviewer.transformCoordinates(SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT, SCENECOORDINATESYSTEM_WORLD, self._sceneviewer.getScene(), in_coords)
+        result, out_coords = self._sceneviewer.transformCoordinates(SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT, SCENECOORDINATESYSTEM_WORLD, self._sceneviewer.getScene(),
+                                                                    in_coords)
         if result == RESULT_OK:
             return out_coords  # [out_coords[0] / out_coords[3], out_coords[1] / out_coords[3], out_coords[2] / out_coords[3]]
 
@@ -351,7 +321,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
 
     def _getNearestGraphic(self, x, y, domain_type):
         self._scenepicker.setSceneviewerRectangle(self._sceneviewer, SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT,
-            x - self._selectTol, y - self._selectTol, x + self._selectTol, y + self._selectTol)
+                                                  x - self._selectTol, y - self._selectTol, x + self._selectTol, y + self._selectTol)
         nearest_graphics = self._scenepicker.getNearestGraphics()
         if nearest_graphics.isValid() and nearest_graphics.getFieldDomainType() == domain_type:
             return nearest_graphics
@@ -383,7 +353,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
 
     def getNearestNode(self, x, y):
         self._scenepicker.setSceneviewerRectangle(self._sceneviewer, SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT,
-            x - self._selectTol, y - self._selectTol, x + self._selectTol, y + self._selectTol)
+                                                  x - self._selectTol, y - self._selectTol, x + self._selectTol, y + self._selectTol)
         node = self._scenepicker.getNearestNode()
 
         return node
@@ -392,7 +362,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         self._scenepicker.addPickedNodesToFieldGroup(selection_group)
 
     def setIgnoreMouseEvents(self, value):
-        self._ignore_mouse_events = value
+        self._handle_mouse_events = not value
 
     def viewAll(self):
         """
@@ -404,7 +374,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
     # paintGL start
     def paintGL(self):
         """
-        Render the scene for this scene viewer.  The QGLWidget has already set up the
+        Render the scene for this scene viewer.  The QOpenGLWidget has already set up the
         correct OpenGL buffer for us so all we need do is render into it.  The scene viewer
         will clear the background so any OpenGL drawing of your own needs to go after this
         API call.
@@ -418,21 +388,23 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         repaint required event all other events are ignored.
         """
         if event.getChangeFlags() & Sceneviewerevent.CHANGE_FLAG_REPAINT_REQUIRED:
-            QtCore.QTimer.singleShot(0, self.updateGL)
+            QtCore.QTimer.singleShot(0, self.update)
 
-#  Not applicable at the current point in time.
-#     def _zincSelectionEvent(self, event):
-#         print(event.getChangeFlags())
-#         print('go the selection change')
+    #  Not applicable at the current point in time.
+    #     def _zincSelectionEvent(self, event):
+    #         print(event.getChangeFlags())
+    #         print('go the selection change')
 
     # resizeGL start
     def resizeGL(self, width, height):
         """
         Respond to widget resize events.
         """
-        self._sceneviewer.setViewportSize(width, height)
+        if self._sceneviewer:
+            pixel_scale = self.window().devicePixelRatio()
+            self._sceneviewer.setViewportSize(width * pixel_scale, height * pixel_scale)
         # resizeGL end
-        
+
     def keyPressEvent(self, event):
         if self._selectionKeyHandling and (event.key() == QtCore.Qt.Key_S) and (event.isAutoRepeat() == False):
             self._selectionKeyPressed = True
@@ -452,17 +424,17 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         Handle a mouse press event in the scene viewer.
         """
         self._use_zinc_mouse_event_handling = False  # Track when zinc should be handling mouse events
-        if self._ignore_mouse_events:
+        if not self._handle_mouse_events:
             event.ignore()
             return
 
         event.accept()
-        if event.button() not in button_map:
+        if event.button() not in BUTTON_MAP:
             return
-        
+
         self._selection_position_start = (event.x(), event.y())
 
-        if button_map[event.button()] == Sceneviewerinput.BUTTON_TYPE_LEFT\
+        if BUTTON_MAP[event.button()] == Sceneviewerinput.BUTTON_TYPE_LEFT \
                 and self._selectionKeyPressed and (self._nodeSelectMode or self._elemSelectMode):
             self._selection_mode = SelectionMode.EXCLUSIVE
             if event.modifiers() & QtCore.Qt.SHIFT:
@@ -471,8 +443,9 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
             scene_input = self._sceneviewer.createSceneviewerinput()
             scene_input.setPosition(event.x(), event.y())
             scene_input.setEventType(Sceneviewerinput.EVENT_TYPE_BUTTON_PRESS)
-            scene_input.setButtonType(button_map[event.button()])
+            scene_input.setButtonType(BUTTON_MAP[event.button()])
             scene_input.setModifierFlags(modifier_map(event.modifiers()))
+            self.makeCurrent()
             self._sceneviewer.processSceneviewerinput(scene_input)
             self._use_zinc_mouse_event_handling = True
 
@@ -480,118 +453,124 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         """
         Handle a mouse release event in the scene viewer.
         """
-        if self._ignore_mouse_events:
+        if not self._handle_mouse_events:
             event.ignore()
             return
         event.accept()
 
-        if event.button() not in button_map:
+        if event.button() not in BUTTON_MAP:
             return
 
         if self._selection_mode != SelectionMode.NONE:
             self._removeSelectionBox()
-            x = event.x()
-            y = event.y()
-            # Construct a small frustum to look for nodes in.
-            scene = self._sceneviewer.getScene()
-            region = scene.getRegion()
-            region.beginHierarchicalChange()
 
             scenepicker = self.getScenepicker()
-            if (x != self._selection_position_start[0]) or (y != self._selection_position_start[1]):
-                # box select
-                left = min(x, self._selection_position_start[0])
-                right = max(x, self._selection_position_start[0])
-                bottom = min(y, self._selection_position_start[1])
-                top = max(y, self._selection_position_start[1])
-                scenepicker.setSceneviewerRectangle(self._sceneviewer, SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT, left, bottom, right, top);
-                if self._selection_mode == SelectionMode.EXCLUSIVE:
-                    self.clearSelection()
-                if self._nodeSelectMode or self._dataSelectMode or self._elemSelectMode:
-                    selectionGroup = self.getOrCreateSelectionGroup()
-                    if self._nodeSelectMode or self._dataSelectMode:
-                        scenepicker.addPickedNodesToFieldGroup(selectionGroup)
-                    if self._elemSelectMode:
-                        scenepicker.addPickedElementsToFieldGroup(selectionGroup)
-
-            else:
-                # point select - get nearest object only
-                scenepicker.setSceneviewerRectangle(self._sceneviewer, SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT,
-                    x - self._selectTol, y - self._selectTol, x + self._selectTol, y + self._selectTol)
-                nearestGraphics = scenepicker.getNearestGraphics()
-                if (self._nodeSelectMode or self._dataSelectMode or self._elemSelectMode) \
-                        and (self._selection_mode == SelectionMode.EXCLUSIVE) \
-                        and not nearestGraphics.isValid():
-                    self.clearSelection()
-
-                if (self._nodeSelectMode and (nearestGraphics.getFieldDomainType() == Field.DOMAIN_TYPE_NODES)) or \
-                    (self._dataSelectMode and (nearestGraphics.getFieldDomainType() == Field.DOMAIN_TYPE_DATAPOINTS)):
-                    node = scenepicker.getNearestNode()
-                    if node.isValid():
-                        nodeset = node.getNodeset()
+            if scenepicker:
+                x = event.x()
+                y = event.y()
+                # Construct a small frustum to look for nodes in.
+                scene = self._sceneviewer.getScene()
+                region = scene.getRegion()
+                region.beginHierarchicalChange()
+                if (x != self._selection_position_start[0]) or (y != self._selection_position_start[1]):
+                    # box select
+                    left = min(x, self._selection_position_start[0])
+                    right = max(x, self._selection_position_start[0])
+                    bottom = min(y, self._selection_position_start[1])
+                    top = max(y, self._selection_position_start[1])
+                    scenepicker.setSceneviewerRectangle(self._sceneviewer, SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT, left, bottom, right, top);
+                    if self._selection_mode == SelectionMode.EXCLUSIVE:
+                        self.clearSelection()
+                    if self._nodeSelectMode or self._dataSelectMode or self._elemSelectMode:
                         selectionGroup = self.getOrCreateSelectionGroup()
-                        nodegroup = selectionGroup.getFieldNodeGroup(nodeset)
-                        if not nodegroup.isValid():
-                            nodegroup = selectionGroup.createFieldNodeGroup(nodeset)
-                        group = nodegroup.getNodesetGroup()
-                        if self._selection_mode == SelectionMode.EXCLUSIVE:
-                            remove_current = (group.getSize() == 1) and group.containsNode(node)
-                            selectionGroup.clear()
-                            if not remove_current:
-                                # re-find node group lost by above clear()
-                                nodegroup = selectionGroup.getFieldNodeGroup(nodeset)
-                                if not nodegroup.isValid():
-                                    nodegroup = selectionGroup.createFieldNodeGroup(nodeset)
-                                group = nodegroup.getNodesetGroup()
-                                group.addNode(node)
-                        elif self._selection_mode == SelectionMode.ADDITIVE:
-                            if group.containsNode(node):
-                                group.removeNode(node)
-                            else:
-                                group.addNode(node)
+                        if self._nodeSelectMode or self._dataSelectMode:
+                            scenepicker.addPickedNodesToFieldGroup(selectionGroup)
+                        if self._elemSelectMode:
+                            scenepicker.addPickedElementsToFieldGroup(selectionGroup)
 
-                if self._elemSelectMode and (nearestGraphics.getFieldDomainType() in \
-                        [Field.DOMAIN_TYPE_MESH1D, Field.DOMAIN_TYPE_MESH2D, Field.DOMAIN_TYPE_MESH3D, Field.DOMAIN_TYPE_MESH_HIGHEST_DIMENSION]):
-                    elem = scenepicker.getNearestElement()
-                    if elem.isValid():
-                        mesh = elem.getMesh()
-                        selectionGroup = self.getOrCreateSelectionGroup()
-                        elementgroup = selectionGroup.getFieldElementGroup(mesh)
-                        if not elementgroup.isValid():
-                            elementgroup = selectionGroup.createFieldElementGroup(mesh)
-                        group = elementgroup.getMeshGroup()
-                        if self._selection_mode == SelectionMode.EXCLUSIVE:
-                            remove_current = (group.getSize() == 1) and group.containsElement(elem)
-                            selectionGroup.clear()
-                            if not remove_current:
-                                # re-find element group lost by above clear()
-                                elementgroup = selectionGroup.getFieldElementGroup(mesh)
-                                if not elementgroup.isValid():
-                                    elementgroup = selectionGroup.createFieldElementGroup(mesh)
-                                group = elementgroup.getMeshGroup()
-                                group.addElement(elem)
-                        elif self._selection_mode == SelectionMode.ADDITIVE:
-                            if group.containsElement(elem):
-                                group.removeElement(elem)
-                            else:
-                                group.addElement(elem)
+                else:
+                    # point select - get nearest object only
+                    scenepicker.setSceneviewerRectangle(self._sceneviewer, SCENECOORDINATESYSTEM_WINDOW_PIXEL_TOP_LEFT,
+                                                        x - self._selectTol, y - self._selectTol, x + self._selectTol, y + self._selectTol)
+                    nearestGraphics = scenepicker.getNearestGraphics()
+                    if (self._nodeSelectMode or self._dataSelectMode or self._elemSelectMode) \
+                            and (self._selection_mode == SelectionMode.EXCLUSIVE) \
+                            and not nearestGraphics.isValid():
+                        self.clearSelection()
+
+                    if (self._nodeSelectMode and (nearestGraphics.getFieldDomainType() == Field.DOMAIN_TYPE_NODES)) or \
+                            (self._dataSelectMode and (nearestGraphics.getFieldDomainType() == Field.DOMAIN_TYPE_DATAPOINTS)):
+                        node = scenepicker.getNearestNode()
+                        if node.isValid():
+                            nodeset = node.getNodeset()
+                            selectionGroup = self.getOrCreateSelectionGroup()
+                            nodegroup = selectionGroup.getFieldNodeGroup(nodeset)
+                            if not nodegroup.isValid():
+                                nodegroup = selectionGroup.createFieldNodeGroup(nodeset)
+                            group = nodegroup.getNodesetGroup()
+                            if self._selection_mode == SelectionMode.EXCLUSIVE:
+                                remove_current = (group.getSize() == 1) and group.containsNode(node)
+                                selectionGroup.clear()
+                                if not remove_current:
+                                    # re-find node group lost by above clear()
+                                    nodegroup = selectionGroup.getFieldNodeGroup(nodeset)
+                                    if not nodegroup.isValid():
+                                        nodegroup = selectionGroup.createFieldNodeGroup(nodeset)
+                                    group = nodegroup.getNodesetGroup()
+                                    group.addNode(node)
+                            elif self._selection_mode == SelectionMode.ADDITIVE:
+                                if group.containsNode(node):
+                                    group.removeNode(node)
+                                else:
+                                    group.addNode(node)
+
+                    if self._elemSelectMode and \
+                            (nearestGraphics.getFieldDomainType() in
+                             [Field.DOMAIN_TYPE_MESH1D, Field.DOMAIN_TYPE_MESH2D, Field.DOMAIN_TYPE_MESH3D, Field.DOMAIN_TYPE_MESH_HIGHEST_DIMENSION]):
+                        elem = scenepicker.getNearestElement()
+                        if elem.isValid():
+                            mesh = elem.getMesh()
+                            selectionGroup = self.getOrCreateSelectionGroup()
+                            elementgroup = selectionGroup.getFieldElementGroup(mesh)
+                            if not elementgroup.isValid():
+                                elementgroup = selectionGroup.createFieldElementGroup(mesh)
+                            group = elementgroup.getMeshGroup()
+                            if self._selection_mode == SelectionMode.EXCLUSIVE:
+                                remove_current = (group.getSize() == 1) and group.containsElement(elem)
+                                selectionGroup.clear()
+                                if not remove_current:
+                                    # re-find element group lost by above clear()
+                                    elementgroup = selectionGroup.getFieldElementGroup(mesh)
+                                    if not elementgroup.isValid():
+                                        elementgroup = selectionGroup.createFieldElementGroup(mesh)
+                                    group = elementgroup.getMeshGroup()
+                                    group.addElement(elem)
+                            elif self._selection_mode == SelectionMode.ADDITIVE:
+                                if group.containsElement(elem):
+                                    group.removeElement(elem)
+                                else:
+                                    group.addElement(elem)
+                region.endHierarchicalChange()
 
-            region.endHierarchicalChange()
             self._selection_mode = SelectionMode.NONE
 
         elif self._use_zinc_mouse_event_handling:
             scene_input = self._sceneviewer.createSceneviewerinput()
             scene_input.setPosition(event.x(), event.y())
             scene_input.setEventType(Sceneviewerinput.EVENT_TYPE_BUTTON_RELEASE)
-            scene_input.setButtonType(button_map[event.button()])
+            scene_input.setButtonType(BUTTON_MAP[event.button()])
+            # self.makeCurrent()
             self._sceneviewer.processSceneviewerinput(scene_input)
+            # self._handle_mouse_events = False
+        else:
+            event.ignore()
 
     def mouseMoveEvent(self, event):
         """
         Handle a mouse move event in the scene viewer.
         Behaviour depends on modes set in original mouse press event.
         """
-        if self._ignore_mouse_events:
+        if not self._handle_mouse_events:
             event.ignore()
             return
 
@@ -616,7 +595,10 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
             scene_input.setEventType(Sceneviewerinput.EVENT_TYPE_MOTION_NOTIFY)
             if event.type() == QtCore.QEvent.Leave:
                 scene_input.setPosition(-1, -1)
+            self.makeCurrent()
             self._sceneviewer.processSceneviewerinput(scene_input)
+        else:
+            event.ignore()
 
     def _addUpdateSelectionBox(self, xdiff, ydiff, xoff, yoff):
         # Using a non-ideal workaround for creating a rubber band for selection.
@@ -633,7 +615,7 @@ class SceneviewerWidget(QtOpenGL.QGLWidget):
         attributes.setGlyphShapeType(Glyph.SHAPE_TYPE_CUBE_WIREFRAME)
         attributes.setBaseSize([xdiff, ydiff, 0.999])
         attributes.setGlyphOffset([xoff, -yoff, 0])
-        #self._selectionBox.setVisibilityFlag(True)
+        # self._selectionBox.setVisibilityFlag(True)
         scene.endChange()
 
     def _removeSelectionBox(self):
